/*
 * main.c
 *
 * Table-file generator. Written in gcc in linux, but should work on most platforms.
 */

#include <stdio.h>
#include <math.h>

#define CIRCLE		256
#define QUARTER		(CIRCLE/4)
#define HALF		(CIRCLE/2)
#define QUARTER3	(HALF + QUARTER)
#define FOV			64
#define BYTE_SIZE	256
#define WALL_HEIGHT	320

int	tantbl[CIRCLE + QUARTER],
	dtbl[CIRCLE + QUARTER],
	fishtbl[FOV],
	fhtbl[BYTE_SIZE / 2],
	shtbl[BYTE_SIZE],
	vstbl[BYTE_SIZE],
	vtbl[BYTE_SIZE],
	sintbl[40];

FILE	*file;

int main() {
	int	i, j;
	float	tang;

	/* Create tables */

	for (i = 0; i < CIRCLE; i++) {
		tang = tan(i * M_PI / HALF);
		if (tang > 100000) tang = 0;

		tantbl[i] = (int)(256 * tang);
		if (i > QUARTER && i < QUARTER3) tantbl[i] = -tantbl[i];

		dtbl[i] = (int)(256 * sqrt(tang * tang + 1));
	}
	for (i = CIRCLE; i < CIRCLE + QUARTER; i++) {
		tang = tan(i * M_PI / HALF);
		tantbl[i] = (int)(256 * tang);
		dtbl[i] = (int)(256 * sqrt(tang * tang + 1));
	}

	for (i = -FOV/2; i < FOV/2; i++)
		fishtbl[i + FOV/2] = (int)(255 * cos(i * M_PI / HALF) + 0.5);

	for (i = 0; i < BYTE_SIZE / 2; i++) {
		fhtbl[i] = (int)(WALL_HEIGHT / (0.25f*i + 1));
		if (fhtbl[i] > 127) fhtbl[i] = 127;
		if (!fhtbl[i]) fhtbl[i] = 1;
	}

	for (i = 0; i < BYTE_SIZE; i++) {
		shtbl[i] = (int)(WALL_HEIGHT / (3*i + 33));
		if (shtbl[i] > 127) shtbl[i] = 127;
		if (!shtbl[i]) shtbl[i] = 1;
	}

	for (i = 0; i < 40; i++) sintbl[i] = (int)(32 * sin(i * M_PI / 16));

	for (i = 0; i < BYTE_SIZE; i++) {
		vstbl[i] = (int)(65535.0f / (i + 1));
		if (i <= 64) vtbl[i] = 0;
		else vtbl[i] = 0.5 * (i - 64) * (65535.0f / (i + 1));
	}

	/* Write tables */
	if (!(file = fopen("tbls.inc", "w"))) return 0;

	fprintf(file, "tracetbl:");

	j = 0;
	for (i = 0; i < CIRCLE + QUARTER; i++) {
		if (!j) fprintf(file, "\n\t.dw\t");
		fprintf(file, "%d, %d", tantbl[i], dtbl[i]);
		if (j < 7) fprintf(file, ", ");
		j = (j + 1) & 7;
	}

	fprintf(file, "\n\nfishtbl:");

	j = 0;
	for (i = 0; i < FOV; i++) {
		if (!j) fprintf(file, "\n\t.db\t");
		fprintf(file, "%d", fishtbl[i]);
		if (j < 7) fprintf(file, ", ");
		j = (j + 1) & 7;
	}

	fprintf(file, "\n\nheights:");

	j = 0;
	for (i = 0; i < BYTE_SIZE / 2; i++) {
		if (!j) fprintf(file, "\n\t.db\t");
		fprintf(file, "%d", fhtbl[i]);
		if (j < 15) fprintf(file, ", ");
		j = (j + 1) & 15;
	}

	j = 0;
	for (i = 0; i < BYTE_SIZE / 2; i++) {
		if (!j) fprintf(file, "\n\t.db\t");
		fprintf(file, "%d", shtbl[i]);
		if (j < 15) fprintf(file, ", ");
		j = (j + 1) & 15;
	}

	fprintf(file, "\n\nvtbl:");
	j = 0;
	for (i = 0; i < BYTE_SIZE; i++) {
		if (!j) fprintf(file, "\n\t.dw\t");
		fprintf(file, "%d, %d", vstbl[i], vtbl[i]);
		if (j < 7) fprintf(file, ", ");
		j = (j + 1) & 7;
	}

	fprintf(file, "\n\nsintbl:");

	j = 0;
	for (i = 0; i < 40; i++) {
		if (!j) fprintf(file, "\n\t.db\t");
		fprintf(file, "%d", sintbl[i]);
		if (j < 7) fprintf(file, ", ");
		j = (j + 1) & 7;
	}

	fprintf(file, "\n\nmasks:\n");
	fprintf(file, "\t.db\t128, 64, 32, 16, 8, 4, 2, 1");

	fclose(file);

	return 0;	
}
